%%
%%  MIndiGolog procedures
%%
%%  The following are a collection of useful procedures in the domain,
%%  from which larger programs can be built.
%%
%   Author: Xiaohui Wu, Xi Liang and Wade Huang
%   Date Created: 17/05/08
%
%


proc(cookProcs([], AgeUtensil),
pi(agt,?prim_agent(agt,chopper): pi(obj, ?obj_is_type(obj,tomato)

                      : acquire_object(agt,obj)
							 : release_object(agt,obj)
							 ))
).

proc(cookProcs([H|T], AgtUtensil),
            cookProcs(T,AgtUtensil)
			//pi(myPlate, ?obj_is_type(myPlate, plate)
            : takeCont(AgtUtensil, myPlate)
            : make(H,myPlate): eatDish(myPlate) : washDish(myPlate)
            : return_cont(AgtUtensil, myPlate)
            )
		  
).






%%  Ensure that the agent has control of an object
proc(ensureHas(Agt,Obj),
     if(has_object(Agt,Obj,now),nil,acquire_object(Agt,Obj))
    ).

%%  Carry out the necessary sequence of actions to place one object
%%  inside another, releasing the destination container when done.
proc(doPlaceIn(Agt,Obj,Dest),
     ensureHas(Agt,Obj) // ensureHas(Agt,Dest)
     : place_in(Agt,Obj,Dest)
     : release_object(Agt,Dest)
    ).

%%  Nondeterministically select an object of a given type, gain control
%%  of it, and place it inside a container object.
proc(doPlaceTypeIn(Agt,Type,Dest),
     pi(obj,?obj_is_type(obj,Type)
            : acquire_object(Agt,obj)
            : doPlaceIn(Agt,obj,Dest))
    ).
proc(checkResource(Agt,List), pi(obj, ?checkResource(Agt, List))).

%%  Carry out the necessary actions to transfer the contents of one
%%  container to another, relasing both when finished.
proc(doTransfer(Agt,Source,Dest),
     ensureHas(Agt,Source) // ensureHas(Agt,Dest)
     : transfer(Agt,Source,Dest)
     : release_object(Agt,Source) // release_object(Agt,Dest)
    ).



%%  Chop the given item then place it in the given container.
%%  Releases control of the container when done.  An empty chopping
%%  board is selected nondeterministically.
proc(doChopInto(Agt,Obj,Dest),
     ensureHas(Agt,Obj)
     : pi(myBoard, ?and(obj_is_type(myBoard,board),neg(contents(myBoard,_,now)))
                   : ensureHas(Agt,myBoard)
                   : place_in(Agt,Obj,myBoard)
                   : begin_task(Agt,chop(myBoard))
                   : end_task(Agt,chop(myBoard))
                   : ensureHas(Agt,Dest)
                   : transfer(Agt,myBoard,Dest)
                   : release_object(Agt,myBoard) // release_object(Agt,Dest)
         )
    ).


%%  Make a salad in the specified container.  This involves selecting
%%  appropriate vegetables, chopping them, placing them in the container,
%%  and assorting briefly.
proc(make(salad, Dest),
       pi(agt,?prim_agent(agt,chopper): pi(obj, ?obj_is_type(obj,lettuce)

                      : acquire_object(agt,obj)
                      : doChopInto(agt,obj,Dest)
							 
                )
         )
       //
       pi(agt,?prim_agent(agt,chopper): pi(obj, ?obj_is_type(obj,tomato)

                      : acquire_object(agt,obj)
                      : doChopInto(agt,obj,Dest)
							 
                )
         )
    : pi(agt,?prim_agent(agt,assorter): ensureHas(agt,Dest)
              : begin_task(agt,assort(Dest))
              : end_task(agt,assort(Dest))
              : release_object(agt,Dest)
        )
    ).

%%  Make a steak in the specified container.  This involves selecting
%%  appropriate vegetables, chopping them, placing them in the container,
%%  and assorting briefly.

proc(make(steak, Dest),
       pi(agt,?prim_agent(agt,chopper): pi(obj, ?obj_is_type(obj,onion)
                      : acquire_object(agt,obj)
                     : doChopInto(agt,obj,Dest)
                )
         )
       //
       pi(agt,?prim_agent(agt,chopper): pi(obj, ?obj_is_type(obj,beef)
                      : acquire_object(agt,obj)
                      : doChopInto(agt,obj,Dest)
                )
         )
     : pi(agt,?prim_agent(agt,roaster): pi(myOven, ?obj_is_type(myOven,oven)
     			   					: acquire_object(agt,myOven)
                      : ensureHas(agt,Dest)
                      : place_in(agt,Dest,myOven)
			       					: set_timer(agt,roastTimer,20)
		      	 					: ring_timer(roastTimer)
			       					: doTransfer(agt,myOven,Dest)
		       			)
          )  
		  : pi(agt,?prim_agent(agt,assorter): ensureHas(agt,Dest)
		       		: begin_task(agt,assort(Dest))
		       		: end_task(agt,assort(Dest))
		       		: release_object(agt,Dest)
		      )
    ).

%%
%   wash the specified dishes
%
%   Firstly get the dishes need to wash and then put them into a sink
%   and finally place them back to cupboard.
%
proc(washDish(Source),
       pi(agt,?prim_agent(agt,dishwasher): pi(mySink, ?obj_is_type(mySink,sink)
                      : acquire_object(agt,mySink)
                      : acquire_object(agt,Source)
                      : place_in(agt,Source,mySink)
			       					: set_timer(agt,washTimer,5)
		      	 					: ring_timer(washTimer) 
											: pi(myCupboard, ?obj_is_type(myCupboard,cupboard)
		      	 							   : acquire_object(agt,myCupboard) 
		      	 							   : doTransfer(agt,mySink,myCupboard)        
		      	 					) 
		      	 					: release_object(agt,Source)
                )
         )
    ).

%%
% consume the dishes simulated by releasing the dishes and resources.
%
proc(eatDish(Source),
       pi(agt,?prim_agent(agt,guest): pi(myMainplate, ?obj_is_type(myMainplate,mainplate)
                      : acquire_object(agt,myMainplate)
                      : acquire_object(agt,Source)
                      : place_in(agt,Source,myMainplate)
			       					: set_timer(agt,eatTimer,30)
		      	 					: ring_timer(eatTimer) 
		      	 					: release_object(agt,myMainplate)
		      	 					: release_object(agt,Source)
                )
         )
    ).
	 


%%  Main control program - prepare a nice meal
%   1. take the order
%   2. check the availability and capacity of kitchen
%   3. start to cook.
proc(control,
        pi(waiterAgt, ?prim_agent(waiterAgt, waiter)
        : pi(list, ?read_in(waiterAgt,list)
        : pi(resManAgt, checkResource(resManAgt,list))
        : pi(agt,?prim_agent(agt,utensilManager): cookProcs(list,agt))
            )
        )
    ).

